% This script creates a testable
% MARTe2 cfg file for the cfs1 algorithm
% with these specifications:
%
% tunable parameters MDSplus link via MDSObjLoader instatiation
% tunable waveforms  MDSplus link via MDSObjWavegen instantiation
% single realtime thread, LinuxTimer synched at 1kHz
% algorithms output MDSplus link via standard MDSWriter

%% Parameters
% source MDSplus shotnumber
mdssrcshot=-1;
% destination MDSplus shotnumber (-1 stands for increment current by 1)
mdsdstshot=-1;
% verbosity level
verbosity=2;
% MDS writer period
%mdsperiod=1;
mdsperiod=0.001;

startpath=pwd;
fcnpath=fileparts(mfilename('fullpath'));
eval(sprintf('cd %s',fcnpath));

%% MDSParameters part generation
system('rm -rf mdsparameters.cfg');
diary mdsparameters.cfg
obj.printMARTe2parconfig(-1);
diary off
system('sed -i ''s/obj.printMARTe2parconfig(-1);//g'' mdsparameters.cfg');
system('sed -i ''s/MDSParameters/MDSParamLoader1/g'' mdsparameters.cfg');

%% MDSWavegen datasource part generation
system('rm -rf mdswavegen.cfg');
diary mdswavegen.cfg
wg0dim=obj.printMARTe2wgbusconfig(-1,'algo_cfs1_inBus',1000);
diary off
system('sed -i ''s/obj.printMARTe2wgbusconfig(-1,''algo_cfs1_inBus'',1000);//g'' mdswavegen.cfg');
system('sed -i ''s/MDSWavegen_algo_cfs1_inBus_1/MDSWavegen_0/g'' mdswavegen.cfg');
system('sed -i ''s/wavegen_algo_cfs1_inBus_1/wavegen_0/g'' mdswavegen.cfg');

%% Input bus expansion
system('rm -rf inputbusexp.cfg');
inbus=Simulink.data.evalinGlobal('algo_cfs1','algo_cfs1_inBus');
diary inputbusexp.cfg
insignals=bus2marte2cfg(inbus,'input','DDB1');
diary off
system('sed -i ''s/bus2marte2cfg(inbus,''input'',''DDB1'');//g'' inputbusexp.cfg');
clear inbus

%% Output bus expansion
system('rm -rf outputbusexp.cfg');
outbus=Simulink.data.evalinGlobal('algo_cfs1','algo_cfs1_outBus');
diary outputbusexp.cfg
outsignals=bus2marte2cfg(outbus,'output','DDB1');
diary off
system('sed -i ''s/bus2marte2cfg(outbus,''output'',''DDB1'');//g'' outputbusexp.cfg');
clear outbus

%% Wavegen input stubs
system('rm -rf wavegenbroker1.cfg');
diary wavegenbroker1.cfg
for i=1:numel(insignals)
    fprintf("%s = { DataSource = DDB1 Type = %s }\n", char(insignals{i}{1}),char(insignals{i}{2}));
end
diary off

%% StorageBroker output stubs
system('rm -rf storagebroker1.cfg');
diary storagebroker1.cfg
for i=1:numel(outsignals)
    fprintf("%s = { DataSource = DDB1 Type = %s }\n", char(outsignals{i}{1}),char(outsignals{i}{2}));
end
diary off
system('rm -rf storagebroker2.cfg');
diary storagebroker2.cfg
for i=1:numel(outsignals)
    fprintf("%s = { DataSource = MDSWriter_0 Type = %s }\n", char(outsignals{i}{1}),char(outsignals{i}{2}));
end
diary off

%% MDSWriter stubs
system('rm -rf mdswriter1.cfg');
diary mdswriter1.cfg
for i=1:numel(outsignals)
    fprintf("%s = { NodeName =""cfs1.outputs.%s"" Period = MDSW0PERIOD AutomaticSegmentation = 0 MakeSegmentAfterNWrites = 2000 SamplePhase = 0 } \n", char(outsignals{i}{1}),char(outsignals{i}{5}));
end
diary off

%% cfg build
system('cp algo_cfs1.cfgsrc algo_cfs1.cfg');
system('sed -i -e "/>MDSPARAMS</r mdsparameters.cfg" -e "/>MDSPARAMS</d" algo_cfs1.cfg');
system('sed -i -e "/>RTAPPPRESL</r rtapp-presl.cfgsrc" -e "/>RTAPPPRESL</d" algo_cfs1.cfg');
system('sed -i -e "/>RTAPPSL</r rtapp-simulinkwrapper.cfgsrc" -e "/>RTAPPSL</d" algo_cfs1.cfg');
system('sed -i -e "/>RTAPPPOSTSL</r rtapp-postsl.cfgsrc" -e "/>RTAPPPOSTSL</d" algo_cfs1.cfg');
system('sed -i -e "/>DATASTATES</r data_states.cfgsrc" -e "/>DATASTATES</d" algo_cfs1.cfg');
system('sed -i -e "/>INPUTBUSFLAT</r wavegenbroker1.cfg" -e "/>INPUTBUSFLAT</d" algo_cfs1.cfg');
system('sed -i -e "/>INPUTBUSSTRUCT</r inputbusexp.cfg" -e "/>INPUTBUSSTRUCT</d" algo_cfs1.cfg');
system('sed -i -e "/>OUTPUTBUSSTRUCT</r outputbusexp.cfg" -e "/>OUTPUTBUSSTRUCT</d" algo_cfs1.cfg');
system('sed -i -e "/>OUTPUTBUSFLAT</r storagebroker1.cfg" -e "/>OUTPUTBUSFLAT</d" algo_cfs1.cfg');
system('sed -i -e "/>OUTPUTBUSFLATMDS</r storagebroker2.cfg" -e "/>OUTPUTBUSFLATMDS</d" algo_cfs1.cfg');
system('sed -i -e "/>MDSWAVEGEN0</r mdswavegen.cfg" -e "/>MDSWAVEGEN0</d" algo_cfs1.cfg');
system('sed -i -e "/>OUTPUTBUSMDSWRITER</r mdswriter1.cfg" -e "/>OUTPUTBUSMDSWRITER</d" algo_cfs1.cfg');
system(sprintf('sed -i "s/>WAVEGEN0_ELEMS</%d/g" algo_cfs1.cfg',wg0dim));

%% cfg adapt
system(sprintf('sed -i "s/MDSSRCSHOT/%d/g" algo_cfs1.cfg',mdssrcshot));
system(sprintf('sed -i "s/MDSDSTSHOT/%d/g" algo_cfs1.cfg',mdsdstshot));
system(sprintf('sed -i "s/VERBOSITY/%d/g" algo_cfs1.cfg',verbosity));
system(sprintf('sed -i "s/MDSW0PERIOD/%d/g" algo_cfs1.cfg',mdsperiod));

